%
% dimerisation model for figure 3
%
% WARNING: this takes several minutes to run
%

close all;
clear all;
rng('default');

S1 = 1e5;
S2 = 0;
S3 = 0;

c1 = 1;
c2 = 1/25;
c3 = 1/500;
c4 = 1/2;

S = [S1; S2; S3];
t = 0;

while (t(end)<30)

% compute propensities

  lam1 = c1*S1;
  lam2 = c2*S2;
  lam3 = c3*S1*(S1-1);
  lam4 = c4*S2;

% compute time of next reaction

  lam_sum = lam1+lam2+lam3+lam4;
  dt = - log(rand(1,1))/lam_sum;

% update state based on which reaction has fired

  lam_sum = rand(1,1)*lam_sum;

  if (lam1>lam_sum) 
    S1 = S1-1;
  elseif (lam1+lam2>lam_sum) 
    S2 = S2-1;
    S3 = S3+1;
  elseif (lam1+lam2+lam3>lam_sum) 
    S1 = S1-2;
    S2 = S2+1;
  else
    S1 = S1+2;
    S2 = S2-1;
  end

  S = [S [S1; S2; S3]];
  t = [t t(end)+dt];

  if (floor(t(end))==ceil(t(end-1)))
    fprintf(' t = %d,  #reactions = %7d,  state = %6d %6d %6d \n',...
            floor(t(end)), length(t)-2, S(:,end-1));
  end
end

% re-scale figure for clarity, and set black-and-white plots for paper

figure; pos=get(gcf,'pos'); pos(3:4)=pos(3:4).*[1.5 0.75]; set(gcf,'pos',pos);

set(0,'DefaultAxesColorOrder',[0 0 0]);
set(0,'DefaultAxesLineStyleOrder','-|--|-.');

%
% 2 plots -- one on short timescale and one on long
%

subplot(1,2,1)
plot(t,S(1,:),t,S(2,:),t,S(3,:))
axis([0 0.03 0 1e5])
legend('S_1','S_2','S_3');
title('Transient phase')
xlabel('Time')
ylabel('Copy number')

subplot(1,2,2)
plot(t,S(1,:),t,S(2,:),t,S(3,:))
axis([0 30 0 1e5])
legend('S_1','S_2','S_3');
title('Long phase')
xlabel('Time')
ylabel('Copy number')

%
% save both figure and tabulated data
%

print('-deps2','ctmp_path.eps');

file = fopen('ctmp_path.txt','w');
fprintf(file, '   Tabulated data for figure 3 \n');
fprintf(file, '             t         S1       S2       S3 \n');
for n = 1:length(t)
  fprintf(file, ' %16.12f %8d %8d %8d \n',t(n),S(:,n));
end
fclose(file);
